package eu.ztsh.garmin

import android.Manifest
import android.annotation.SuppressLint
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothManager
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.util.Log
import androidx.activity.result.ActivityResultCallback
import androidx.activity.result.contract.ActivityResultContracts.StartActivityForResult
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.LifecycleOwner
import com.mapbox.navigation.base.formatter.DistanceFormatterOptions
import com.mapbox.navigation.base.options.NavigationOptions
import com.mapbox.navigation.base.trip.model.RouteProgress
import com.mapbox.navigation.core.formatter.MapboxDistanceFormatter
import com.mapbox.navigation.core.lifecycle.MapboxNavigationApp
import com.mapbox.navigation.core.trip.session.RouteProgressObserver
import com.mapbox.navigation.ui.maneuver.api.MapboxManeuverApi
import eu.ztsh.garmin.databinding.ActivityMainBinding

@SuppressLint("MissingPermission")
class MainActivity : AppCompatActivity() {

    lateinit var garmin: Garmin

    private lateinit var binding: ActivityMainBinding
    private val mapboxObserver = MapboxObserver()

    init {
        lifecycle.addObserver(object : DefaultLifecycleObserver {
            override fun onResume(owner: LifecycleOwner) {
                MapboxNavigationApp.attach(owner)
                MapboxNavigationApp.registerObserver(mapboxObserver)
            }

            override fun onPause(owner: LifecycleOwner) {
                MapboxNavigationApp.detach(owner)
                MapboxNavigationApp.unregisterObserver(mapboxObserver)
            }
        })
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)
        if (!MapboxNavigationApp.isSetup()) {
            MapboxNavigationApp.setup {
                NavigationOptions.Builder(applicationContext)
                    .accessToken(BuildConfig.MAPBOX_DOWNLOADS_TOKEN)
                    .build()
            }
        }
        MapboxNavigationApp.current()?.startTripSession()
        bluetoothInit()
    }

    override fun onStart() {
        super.onStart()
        MapboxNavigationApp.current()?.registerRouteProgressObserver(routeProgressObserver)
    }
    override fun onStop() {
        super.onStop()
        MapboxNavigationApp.current()?.unregisterRouteProgressObserver(routeProgressObserver)
    }

    override fun onDestroy() {
        super.onDestroy()
        MapboxNavigationApp.current()?.stopTripSession()
        maneuverApi.cancel()
    }


    // Define distance formatter options
    private val distanceFormatter: DistanceFormatterOptions by lazy {
        DistanceFormatterOptions.Builder(this).build()
    }
    // Create an instance of the Maneuver API
    private val maneuverApi: MapboxManeuverApi by lazy {
        MapboxManeuverApi(MapboxDistanceFormatter(distanceFormatter))
    }

    private val routeProgressObserver =
        RouteProgressObserver { routeProgress -> maneuverApi.getManeuvers(routeProgress).value?.apply { garmin.process(this[0]) } }

    private fun bluetoothInit() {
        val bluetoothManager: BluetoothManager = getSystemService(BluetoothManager::class.java)
        val bluetoothAdapter: BluetoothAdapter = bluetoothManager.adapter
            ?: // Device doesn't support Bluetooth
            throw Exception()
        if (!bluetoothAdapter.isEnabled) {
            // TODO: Start intent
            val enableBtIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
            registerForActivityResult(StartActivityForResult(), ActivityResultCallback { })
        }
        checkBt()

        val pairedDevices: Set<BluetoothDevice>? = bluetoothAdapter.bondedDevices
        val context = this
        pairedDevices?.firstOrNull { device ->
            Log.d(TAG, device.name)
            device.name.equals("GARMIN HUD")
        }?.apply {
            garmin = Garmin(context, this, bluetoothAdapter)
            garmin.start()
        }

    }

    fun checkBt(): Boolean {
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.S) {
            checkBt(Manifest.permission.BLUETOOTH_SCAN)
            checkBt(Manifest.permission.BLUETOOTH_CONNECT)
        } else {
            checkBt(Manifest.permission.BLUETOOTH)
            checkBt(Manifest.permission.BLUETOOTH_ADMIN)
        }
        return true
    }

    private fun checkBt(permission: String): Boolean {
        if (ActivityCompat.checkSelfPermission(
                this,
                permission
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(this, arrayOf(permission), 1)
        }
        return true
    }

    companion object {
        private const val TAG = "bt"
    }

}
